@@ -1,6 +1,6 @@
 /* mswindows.c -- Windows-specific support
-   Copyright (C) 1996-2011, 2014-2015, 2018 Free Software Foundation,
-   Inc.
+   Copyright (C) 1996-2011, 2014-2015, 2018-2021 Free Software
+   Foundation, Inc.
 
 This file is part of GNU Wget.
 
