@@ -171,7 +171,7 @@ struct options
 
   wgint limit_rate;             /* Limit the download rate to this
                                    many bps. */
-  SUM_SIZE_INT quota;           /* Maximum file size to download and
+  wgint quota;                  /* Maximum file size to download and
                                    store. */
 
   bool server_response;         /* Do we print server response? */
