@@ -1,5 +1,6 @@
 /* URL handling.
-   Copyright (C) 1996-2011, 2015, 2018 Free Software Foundation, Inc.
+   Copyright (C) 1996-2011, 2015, 2018-2021 Free Software Foundation,
+   Inc.
 
 This file is part of GNU Wget.
 
@@ -174,8 +175,8 @@ static const unsigned char urlchr_table[256] =
 static void
 url_unescape_1 (char *s, unsigned char mask)
 {
-  char *t = s;                  /* t - tortoise */
-  char *h = s;                  /* h - hare     */
+  unsigned char *t = (unsigned char *) s; /* t - tortoise */
+  unsigned char *h = (unsigned char *) s; /* h - hare     */
 
   for (; *h; h++, t++)
     {
@@ -186,7 +187,7 @@ url_unescape_1 (char *s, unsigned char mask)
         }
       else
         {
-          char c;
+          unsigned char c;
           /* Do nothing if '%' is not followed by two hex digits. */
           if (!h[1] || !h[2] || !(c_isxdigit (h[1]) && c_isxdigit (h[2])))
             goto copychar;
@@ -1461,23 +1462,37 @@ append_uri_pathel (const char *b, const char *e, bool escaped,
                    struct growable *dest)
 {
   const char *p;
+  char buf[1024];
+  char *unescaped = NULL;
   int quoted, outlen;
-
   int mask;
+  int max_length;
+
+  if (!dest)
+    return;
+
   if (opt.restrict_files_os == restrict_unix)
     mask = filechr_not_unix;
   else if (opt.restrict_files_os == restrict_vms)
     mask = filechr_not_vms;
   else
     mask = filechr_not_windows;
+
   if (opt.restrict_files_ctrl)
     mask |= filechr_control;
 
   /* Copy [b, e) to PATHEL and URL-unescape it. */
   if (escaped)
     {
-      char *unescaped;
-      BOUNDED_TO_ALLOCA (b, e, unescaped);
+      size_t len = e - b;
+		if (len < sizeof (buf))
+        unescaped = buf;
+      else
+        unescaped = xmalloc(len + 1);
+
+		memcpy(unescaped, b, len);
+		unescaped[len] = 0;
+
       url_unescape (unescaped);
       b = unescaped;
       e = unescaped + strlen (unescaped);
@@ -1502,8 +1517,25 @@ append_uri_pathel (const char *b, const char *e, bool escaped,
      string length.  Each quoted char introduces two additional
      characters in the string, hence 2*quoted.  */
   outlen = (e - b) + (2 * quoted);
+# ifdef WINDOWS
+  max_length = MAX_PATH;
+# else
+  max_length = get_max_length(dest->base, dest->tail, _PC_NAME_MAX);
+# endif
+  max_length -= CHOMP_BUFFER;
+  if (max_length > 0 && outlen > max_length)
+    {
+      logprintf (LOG_NOTQUIET, "The destination name is too long (%d), reducing to %d\n", outlen, max_length);
+
+      outlen = max_length;
+    }
   GROW (dest, outlen);
 
+  // This should not happen, but it's impossible to argue with static analysis that it can't happen
+  // (in theory it can). So give static analyzers a hint.
+  if (!dest->base)
+    return;
+
   if (!quoted)
     {
       /* If there's nothing to quote, we can simply append the string
@@ -1513,19 +1545,29 @@ append_uri_pathel (const char *b, const char *e, bool escaped,
   else
     {
       char *q = TAIL (dest);
-      for (p = b; p < e; p++)
+      int i;
+
+      for (i = 0, p = b; p < e; p++)
         {
           if (!FILE_CHAR_TEST (*p, mask))
-            *q++ = *p;
-          else
+	    {
+	      if (i == outlen)
+	        break;
+	      *q++ = *p;
+	      i++;
+	    }
+          else if (i + 3 > outlen)
+	    break;
+	  else
             {
               unsigned char ch = *p;
               *q++ = '%';
               *q++ = XNUM_TO_DIGIT (ch >> 4);
               *q++ = XNUM_TO_DIGIT (ch & 0xf);
+	      i += 3;
             }
         }
-      assert (q - TAIL (dest) == outlen);
+      assert (q - TAIL (dest) <= outlen);
     }
 
   /* Perform inline case transformation if required.  */
@@ -1544,6 +1586,9 @@ append_uri_pathel (const char *b, const char *e, bool escaped,
 
   TAIL_INCR (dest, outlen);
   append_null (dest);
+
+  if (unescaped && unescaped != buf)
+	  free (unescaped);
 }
 
 #ifdef HAVE_ICONV
@@ -1568,7 +1613,7 @@ convert_fname (char *fname)
   if (cd == (iconv_t) (-1))
     {
       logprintf (LOG_VERBOSE, _ ("Conversion from %s to %s isn't supported\n"),
-                 quote (from_encoding), quote (to_encoding));
+                 quote_n (0, from_encoding), quote_n (1, to_encoding));
       return fname;
     }
 
@@ -1669,6 +1714,7 @@ append_dir_structure (const struct url *u, struct growable *dest)
 
       if (dest->tail)
         append_char ('/', dest);
+
       append_uri_pathel (pathel, next, true, dest);
     }
 }
@@ -1685,7 +1731,6 @@ url_file_name (const struct url *u, char *replaced_filename)
   const char *u_file;
   char *fname, *unique, *fname_len_check;
   const char *index_filename = "index.html"; /* The default index file is index.html */
-  size_t max_length;
 
   fnres.base = NULL;
   fnres.size = 0;
@@ -1779,41 +1824,8 @@ url_file_name (const struct url *u, char *replaced_filename)
   temp_fnres.size = 0;
   temp_fnres.tail = 0;
   append_string (fname, &temp_fnres);
+
   xfree (fname);
-
-  /* Check that the length of the file name is acceptable. */
-#ifdef WINDOWS
-  if (MAX_PATH > (fnres.tail + CHOMP_BUFFER + 2))
-    {
-      max_length = MAX_PATH - (fnres.tail + CHOMP_BUFFER + 2);
-      /* FIXME: In Windows a filename is usually limited to 255 characters.
-      To really be accurate you could call GetVolumeInformation() to get
-      lpMaximumComponentLength
-      */
-      if (max_length > 255)
-        {
-          max_length = 255;
-        }
-    }
-  else
-    {
-      max_length = 0;
-    }
-#else
-  max_length = get_max_length (fnres.base, fnres.tail, _PC_NAME_MAX) - CHOMP_BUFFER;
-#endif
-  if (max_length > 0 && strlen (temp_fnres.base) > max_length)
-    {
-      logprintf (LOG_NOTQUIET, "The name is too long, %lu chars total.\n",
-          (unsigned long) strlen (temp_fnres.base));
-      logprintf (LOG_NOTQUIET, "Trying to shorten...\n");
-
-      /* Shorten the file name. */
-      temp_fnres.base[max_length] = '\0';
-
-      logprintf (LOG_NOTQUIET, "New name is %s.\n", temp_fnres.base);
-    }
-
   xfree (fname_len_check);
 
   /* The filename has already been 'cleaned' by append_uri_pathel() above.  So,
@@ -1846,7 +1858,7 @@ url_file_name (const struct url *u, char *replaced_filename)
     }
   else
     {
-      unique = unique_name (fname, true);
+      unique = unique_name_passthrough (fname);
       if (unique != fname)
         xfree (fname);
     }
